# -*- coding: utf-8 -*-
"""
Created on Thu Jan  8 17:30:02 2015

@author: JLavin
"""

from nose.tools import assert_equals
#from hedgeye_analysis import hedgeye_analysis_boilerplate as bp
from hedgeye_analysis.hedgeye_analysis_boilerplate import *
import mock

def test_root_dir_1():
    os.getcwd = mock.Mock(return_value='/mydir/shared')
    assert_equals( root_dir(),
                   '/mydir')

def test_root_dir_2():
    os.getcwd = mock.Mock(return_value='/main/mydir/shared')
    assert_equals( root_dir(),
                   '/main/mydir')

def test_root_dir_3():
    os.getcwd = mock.Mock(return_value='/main/mydir')
    assert_equals( root_dir(),
                   '/main/mydir')

def test_shared_dir():
    os.getcwd = mock.Mock(return_value='/mydir/shared')
    assert_equals( shared_dir(),
                   '/mydir/shared')

def test_copied_dir():
    os.getcwd = mock.Mock(return_value='/mydir/shared')
    assert_equals( copied_dir(),
                   '/mydir/copied')

def test_today_string():
    from datetime import date
    assert_equals( today_string(),
                   date.today().strftime("%Y-%m-%d") )

def test_tomorrow_string():
    assert_equals( tomorrow_string(),
                   tomorrow().strftime("%Y-%m-%d") )

def test_tomorrow():
    assert_equals( tomorrow(),
                   today() + datetime.timedelta(days = 1) )



# -*- coding: utf-8 -*-
"""
HedgeyeClinicalTrials

Created on Fri Feb 20 09:19:42 2015

@author: JLavin
"""

from hedgeye_analysis.hedgeye_analysis_boilerplate import *

def pipe_split(x):
    return str(x).split('|')

def to_yr_month(dateval):
    dateval = pd.to_datetime(dateval)
    try:
        return str(dateval.year) + '-' + str(dateval.month).zfill(2)
    except:
        return ""
    
def month_str_to_yyyy_mm(month_str):
    try:
        return to_yr_month(datetime.datetime.strptime(month_str, "%B %Y"))
    except:
        return ""
    
def prepare_trials_df(df):
    df = df[['Sponsor/Collaborators','First Received','Start Date','Completion Date']]
    df['Start Month'] = pd.to_datetime(df['Start Date']).apply(to_yr_month)
    df['Sponsor/Collaborators'] = df[['Sponsor/Collaborators']].applymap(pipe_split)
    df['NumCollaborators'] = df[['Sponsor/Collaborators']].applymap(len)
    df['Completion Month'] = pd.to_datetime(df['Completion Date']).apply(to_yr_month)
    df['Received Month'] = pd.to_datetime(df['First Received']).apply(to_yr_month)
    return df

def all_months(df):
    start_months = set(df['Start Month']) if ('Start Month' in df.columns) else set()
    completion_months = set(df['Completion Month']) if ('Completion Month' in df.columns) else set()
    received_months = set(df['Received Month']) if ('Received Month' in df.columns) else set()
    months = start_months | completion_months | received_months
    months.discard('')
    months.discard('nan-nan')
    return months

def collaborators_count(df):
    collaborators = {}
    for index, row in df[['Sponsor/Collaborators']].iterrows():
        for collab in row['Sponsor/Collaborators']:
            collaborators[collab.strip()] = collaborators.get(collab.strip(), 0) + 1
    return collaborators

def calc_collabs_for_month(trials_df, df, month_col_name='Completion Month', month_col_value=prev_month_year_dash_month_string()):
    collaborators = {}
    for index, row in trials_df.loc[trials_df[month_col_name] == month_col_value].iterrows():
        for collab in row['Sponsor/Collaborators']:
            collaborators[collab.strip()] = collaborators.get(collab.strip(), 0) + 1
    col_name = month_col_value
    new_collab_df = pd.DataFrame.from_records(sorted(collaborators.items(), key=lambda x: -x[1]), index='Collaborator', columns = ['Collaborator', col_name])
    return df.join(new_collab_df)

def calc_completion_months(trials_df, df):
    for month in all_months(trials_df):
        df = calc_collabs_for_month(trials_df, df, 'Completion Month', month)
    return df.fillna(0).reindex_axis(sorted(df.columns), axis=1)

def calc_received_months(trials_df, df):
    for month in all_months(trials_df):
        df = calc_collabs_for_month(trials_df, df, 'Received Month', month)
    return df.fillna(0).reindex_axis(sorted(df.columns), axis=1)

def calc_start_months(trials_df, df):
    for month in all_months(trials_df):
        df = calc_collabs_for_month(trials_df, df, 'Start Month', month)
    return df.fillna(0).reindex_axis(sorted(df.columns), axis=1)

def data_frame_column_prefix(df):
    """
      Takes df column name like 'Received-2013-02' and returns 'Received'
    """
    import re
    # **** This is failing. I'm not sure why. ****
    return re.sub(r"-.*$", '', df.columns[0])

def plot_collaborator(date_type, collaborator, df, start_month='2000-01', end_month=prev_month_year_dash_month_string()):
    # if start_month is not an index value of df, increment start_month and test again
    while start_month not in df:
        start_month = next_month_year_dash_month_string(today = start_month + '-15')
    # if end_month is not an index value of df, decrement end_month and test again
    while end_month not in df:
        end_month = prev_month_year_dash_month_string(today = end_month + '-15')
    title = date_type + 's for ' + collaborator + ' from ' + start_month + ' to ' + end_month
    series = df.loc[collaborator,start_month:end_month]
    series.fillna(0, inplace=True)
    #series.index = series.index.map(lambda x: x.replace(date_type + '-',''))
    series.plot(title=title, figsize=(8,8), ylim = (0, series.max() + 1))

"""
month_col_name can be 'Completion Month' (the default), 'Start Month' or 'Received Month'
month_col_value should be in 'YYYY-MM' format, like '2015-02' 
"""
def chart_top_collabs_for_month(df, month_col_value=prev_month_year_dash_month_string(), num_collaborators=20, date_type=None):
    if date_type:  # should be 'Start,' 'Completion,' or 'Submitted'
      date_type = date_type + ' '
    else:
      date_type = ''
    title = "Top " + str(num_collaborators) + " Collaborators for " + date_type + "Month " + month_col_value
    df.ix[:,[month_col_value]].sort([month_col_value],ascending=[0]).head(20).plot(kind="bar", figsize=(8,8), title=title)




# -*- coding: utf-8 -*-
"""
HedgeyeStockAnalysis
"""

from hedgeye_analysis.hedgeye_analysis_boilerplate import *
#import hedgeye_analysis.hedgeye_pandas_patch

import pandas.io.data as web

def savedir(source):
  return globals()["{}_savedir".format(source)]()

def default_savedir():
  """ Default directory for saving data """
  return shared_dir() + '/data'

def yahoo_savedir():
  """ Default directory for saving data downloaded from Yahoo """
  return shared_dir() + '/data/yahoo'

def fred_savedir():
  """ Default directory for saving data downloaded from FRED """
  return shared_dir() + '/data/fred'

def google_savedir():
  """ Default directory for saving data downloaded from Google """
  return shared_dir() + '/data/google'

""" 
    Takes list of stock dictionaries, like:
        stocks = [{'df_name': 'aal_close',        'ticker': 'AAL'},
                  {'df_name': 'vix_close',        'ticker': '%5EVIX'},
                  {'df_name': 'gold_close',       'ticker': 'GLD'},
                  {'df_name': 'oil_close',        'ticker': 'OIL'},
                  {'df_name': 'gas_close',        'ticker': 'GASPRICE',     'source': 'fred'},
                  {'df_name': 'gcgas_close',      'ticker': 'DGASUSGULF',   'source': 'fred'}]
    where `ticker` is the name of the variable to download,
          `df_name` is the variable the dataframe will be assigned to, and
          `source` is the (optional) data source (by default: 'yahoo')
    For each stock in stocks, runs something like:
        `aal_close = sa.get_stock_close('AAL', source = 'fred')`
    which downloads data from the source, saves it to a CSV file
    in the shared directory, then saves the closing values in the
    `aal_close` variable.
"""
def download_stocks(stocks):
    for index, stock in enumerate(stocks):
        if 'source' in stock.keys():
            exec(stock['df_name'] + " = sa.get_stock_close('" + stock['ticker'] + "', source = '" + stock['source'] + "')")
        else:
            exec(stock['df_name'] + " = sa.get_stock_close('" + stock['ticker'] + "')")

def plot_two_stock_returns(tickers_or_returns_df, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, **kwargs):
  """ Correlation plot of stock returns for the first two tickers (or DataFrame columns) """
  if end is None:
    end = yesterday() 
  returns = _tickers_to_returns_df(tickers_or_returns_df, **kwargs).iloc[:,0:2]
  ax = returns.plot(kind='scatter', x=returns.columns.values[0], y=returns.columns.values[1], alpha=0.1, **kwargs)
  ax.set_xlabel(returns.columns.values[0])
  ax.set_ylabel(returns.columns.values[1])

def plot_prices(tickers_or_prices_df, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, **kwargs):
  """ Plot of stock prices for the first two tickers (or DataFrame columns) """
  if end is None:
    end = yesterday() 
  prices = _tickers_to_adj_close_df(tickers_or_prices_df, **kwargs)
  ax = prices.plot(subplots=True, figsize=(15,1.5*np.shape(prices)[1]), **kwargs)

def plot_returns(tickers_or_returns_df, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, **kwargs):
  """ Plot of stock returns for the first two tickers (or DataFrame columns) """
  if end is None:
    end = yesterday() 
  returns = _tickers_to_returns_df(tickers_or_returns_df, **kwargs)
  ax = returns.plot(subplots=True, figsize=(15,1.5*np.shape(returns)[1]), **kwargs)

def plot_stock_and_volume():
    import urllib
    import time
    import datetime
    import matplotlib.ticker as mticker
    import matplotlib.dates as mdates

def _tickers_to_returns_df(tickers, **kwargs):
  if isinstance(tickers, pd.DataFrame):
    return tickers
  else:
    return returns_df(tickers, **kwargs)

def _tickers_to_adj_close_df(tickers, **kwargs):
  if isinstance(tickers, pd.DataFrame):
    return tickers
  else:
    return adjusted_close_df(tickers, **kwargs)

def reload():
    pass

def load_stock_from_csv(ticker, savefile, source):
    print("Loading stock {} from CSV file".format(ticker))
    return globals()['__read_stock_from_%s' % source](ticker, savefile)

def download_stock_from_source(ticker, savefile, start, end, source):
    print("Downloading stock {} from {}".format(ticker, source))
    return globals()['__get_stock_from_%s' % source](ticker, savefile, start, end)

def get_stock(ticker, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, reload_after_days=2):
    """ Download (or read from file) stock data for ticker, save it, and return it 
        reload = True forces the data to be downloaded, even if it's up-to-date
        reload_after_days specifies how many days prior to today the last observation can be before it will auto-reload """
    if end is None:
        end = yesterday() 
    savefile = savedir(source) + '/' + ticker.upper() + '.csv'
    if (not reload) and file_exists(savefile):
        stock_data = load_stock_from_csv(ticker, savefile, source)
        # ideally, this method would know about weekends and bank holidays
        if stock_data.stale_days() < reload_after_days:
            return stock_data
        else:
            print("Data too stale. Re-downloading...")
            return download_stock_from_source(ticker, savefile, start, end, source)
    else:
        return download_stock_from_source(ticker, savefile, start, end, source)

def get_stock_close(ticker, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, reload_after_days=2):
    """ Download (or read from file) stock data for ticker, save it, and return just the Adjusted Close """
    df = get_stock(ticker, start, end, source, reload, reload_after_days)
    if source == 'yahoo':
        return df['Adj Close']
    elif source == 'fred':
        return df[ticker]
    else:
        raise "#get_stock_close not yet implemented for {}".format(source)

def add_stock_to_df(ticker, df):
    """ Add column of Adjusted Close data for ticker to DataFrame """
    stock = get_stock(ticker)[['Adj Close']].rename(columns={'Adj Close':ticker})
    return df.join(stock)

def returns_df(tickers, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False, **kwargs):
  """ Build DataFrame with one column of %Chg(Close) data per ticker in tickers """
  if end is None:
    end = yesterday() 
  for ticker in tickers:
    tmp = get_stock(ticker, start=start, end=end, source=source, reload=reload)
    stock = pd.DataFrame(tmp['PctChg'])
    stock.columns = [ticker]
    if 'stock_returns' in locals():
      stock_returns = stock_returns.join(stock)    
    else:
      stock_returns = stock
  return stock_returns.dropna()

def adjusted_close_df(tickers, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False):
  """ Build DataFrame with one column of %Chg(Adjusted Close) data per ticker in tickers """
  if end is None:
    end = yesterday() 
  for ticker in tickers:
    tmp = get_stock(ticker, start=start, end=end, source=source, reload=reload)
    stock = pd.DataFrame(tmp['Adj Close'])
    stock.columns = [ticker]
    if 'stock_returns' in locals():
      stock_returns = stock_returns.join(stock)    
    else:
      stock_returns = stock
  return stock_returns

def get_stocks(tickers, start=datetime.datetime(1980,1,1), end=None, source='yahoo', reload=False):
  """ Download stock data for each stock ticker in tickers """
  if end is None:
    end = yesterday() 
  for ticker in tickers:
    get_stock(ticker, start=start, end=end, source=source, reload=reload)

def __read_stock_from_yahoo(ticker, savefile):
  return pd.read_csv(savefile, index_col = 'Date', parse_dates = True)

def __read_stock_from_google(ticker, savefile):
    print("Called with savefile ", savefile)
    return pd.read_csv(savefile, index_col = 'Date', parse_dates = True)

def __read_stock_from_fred(ticker, savefile):
  return pd.read_csv(savefile, index_col = 'DATE', parse_dates = True)

def __read_stock_close_from_yahoo(ticker, savefile):
  tmp = pd.read_csv(savefile, index_col = 'Date', parse_dates = True)
  return tmp['Adj Close']

def __read_stock_close_from_fred(ticker, savefile):
  tmp = pd.read_csv(savefile, index_col = 'DATE', parse_dates = True)
  return tmp[ticker]

def __get_stock_from_yahoo(ticker, savefile, start, end):
  # http://pandas.pydata.org/pandas-docs/stable/remote_data.html
  data = web.DataReader(ticker, 'yahoo', start, end)
  data['PctChg'] = data['Adj Close'].pct_change()
  data.to_csv(savefile)
  return data

def __get_stock_from_google(ticker, savefile, start, end):
  # http://pandas.pydata.org/pandas-docs/stable/remote_data.html
  data = web.DataReader(ticker, 'google', start, end)
  data['PctChg'] = data['Close'].pct_change()
  data.to_csv(savefile)
  return data

def __get_stock_from_fred(series, savefile, start, end):
  # http://pandas.pydata.org/pandas-docs/stable/remote_data.html
  data = web.DataReader(series, 'fred', start, end)
  # Should do something like the following:
  #data['PctChg'] = data['Adj Close'].pct_change()
  data.to_csv(savefile)
  return data

def __get_stock_close_from_yahoo(ticker, savefile, start, end):
  # http://pandas.pydata.org/pandas-docs/stable/remote_data.html
  data = web.DataReader(ticker, 'yahoo', start, end)
  data['PctChg'] = data['Adj Close'].pct_change()
  data.to_csv(savefile)
  return data['Adj Close']

def __get_stock_close_from_fred(series, savefile, start, end):
  # http://pandas.pydata.org/pandas-docs/stable/remote_data.html
  data = web.DataReader(series, 'fred', start, end)
  # Should do something like the following:
  #data['PctChg'] = data['Adj Close'].pct_change()
  data.to_csv(savefile)
  return data[series]



# -*- coding: utf-8 -*-
"""
Created on Wed Jan 28 09:09:21 2015

@author: JLavin
"""

import datetime
import pandas as pd
#import hedgeye_analysis.hedgeye_stock_analysis as sa

def stale_days(self):
    """ Return integer number of calendar days since last observation
        according to dataframe index """
    staleness = datetime.datetime.now() - self.index.to_pydatetime()[-1]
    return staleness.days

pd.DataFrame.stale_days = stale_days

def add_ticker_adj_close(self, ticker):
    """ Add column of Adjusted Close data for ticker to DataFrame """
    stock = globals()['sa'].get_stock(ticker)[['Adj Close']].rename(columns={'Adj Close':ticker})
    #stock = get_stock(ticker)[['Adj Close']].rename(columns={'Adj Close':ticker})
    return self.join(stock)

pd.DataFrame.add_ticker_adj_close = add_ticker_adj_close


import __main__ as main_mod
import datetime
import numpy as np
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn
import os
main_mod.datetime = datetime
main_mod.np = np
main_mod.pd = pd
main_mod.mpl = mpl
main_mod.plt = plt
main_mod.os = os
main_mod.seaborn = seaborn



from __future__ import division
import datetime
from dateutil.relativedelta import relativedelta

import numpy as np
import matplotlib as mpl
from matplotlib import pyplot as plt
import seaborn
import pandas as pd
import os
import os.path
import hedgeye_analysis.hedgeye_pandas_patch
from hedgeye_analysis.introspection import print_source

from IPython.display import display
from IPython.display import HTML

def file_exists(filename):
    return os.path.isfile(filename)

def root_dir():
    curdir = os.getcwd()
    import re
    curdir = re.sub('/copied.*', '', curdir)
    return re.sub('/shared.*', '', curdir)

def shared_dir():
    return root_dir() + '/shared'

def copied_dir():
    return root_dir() + '/copied'

def today_string(today = None):
    """Returns a string representing today

    >>> today_string('2014-03-11')
    '2014-03-11'
    >>> today_string('2014-09-30')
    '2014-09-30'
    >>> today_string('2014-12-31')
    '2014-12-31'
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return today.strftime("%Y-%m-%d")

def year_dash_month_string(today = None):
    """Returns a string representing current month

    >>> year_dash_month_string('2014-03-11')
    '2014-03'
    >>> year_dash_month_string('2014-09-30')
    '2014-09'
    >>> year_dash_month_string('2014-12-31')
    '2014-12'
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return today.strftime("%Y-%m")

def prev_month_year_dash_month_string(today = None):
    """Returns a string representing current month

    >>> prev_month_year_dash_month_string('2014-03-11')
    '2014-02'
    >>> prev_month_year_dash_month_string('2014-09-30')
    '2014-08'
    >>> prev_month_year_dash_month_string('2014-12-31')
    '2014-11'
    >>> prev_month_year_dash_month_string('2015-01-27')
    '2014-12'
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return (today + relativedelta(months = -1)).strftime("%Y-%m")

def next_month_year_dash_month_string(today = None):
    """Returns a string representing the next month

    >>> next_month_year_dash_month_string('2014-03-11')
    '2014-04'
    >>> next_month_year_dash_month_string('2014-09-30')
    '2014-10'
    >>> next_month_year_dash_month_string('2014-12-31')
    '2015-01'
    >>> next_month_year_dash_month_string('2015-01-27')
    '2015-02'
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return (today + relativedelta(months = +1)).strftime("%Y-%m")

def tomorrow_string(today = None):
    """Returns a string representing tomorrow or the day after the day passed in

    >>> tomorrow_string('2014-03-11')
    '2014-03-12'
    >>> tomorrow_string('2014-09-30')
    '2014-10-01'
    >>> tomorrow_string('2014-12-31')
    '2015-01-01'
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return (today + datetime.timedelta(days = 1)).strftime("%Y-%m-%d")

def tomorrow(today = None):
    """ Returns date object representing tomorrow or the day after the day passed in

    >>> tomorrow('2015-06-15')
    datetime.date(2015, 6, 16)
    """
    if today is None:
        today = datetime.date.today()
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return (today + datetime.timedelta(days = 1))

def next_date(date):
    """ deprecated """
    return tomorrow_string(date)
    #return (datetime.datetime.strptime(date,"%Y-%m-%d") + datetime.timedelta(days = 1)).strftime("%Y-%m-%d")

def today():
    return datetime.date.today()

def yesterday(today = today()):
    """Returns date object representing yesterday or the day before the day passed in

    >>> yesterday(datetime.date(2014, 3, 11))
    datetime.date(2014, 3, 10)
    >>> yesterday(datetime.date(2014, 10, 1))
    datetime.date(2014, 9, 30)
    >>> yesterday(datetime.date(2015, 1, 1))
    datetime.date(2014, 12, 31)
    """
    return (today - datetime.timedelta(days = 1))

def yesterday_string(today = today()):
    """Returns a string representing yesterday or the day before the day passed in

    >>> yesterday_string('2014-03-11')
    '2014-03-10'
    >>> yesterday_string('2014-10-01')
    '2014-09-30'
    >>> yesterday_string('2015-01-01')
    '2014-12-31'
    """
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return yesterday(today).strftime("%Y-%m-%d")

def week_ago_string(today = datetime.date.today()):
    """deprecated"""
    #if isinstance(today, str):
    #  today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    #return (today - datetime.timedelta(days = 7)).strftime("%Y-%m-%d")
    return n_weeks_ago_date_string(1, today=today)

def month_ago_string(today = datetime.date.today()):
    """deprecated"""
    #if isinstance(today, str):
    #  today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    #return (today - datetime.timedelta(months = 1)).strftime("%Y-%m-%d")
    return n_months_ago_date_string(1, today=today)

def n_months_ago_date(n, today=datetime.date.today()):
    return (today - relativedelta(months=n))

def n_months_ago_date_string(n, today=datetime.date.today()):
    """Returns a string representing n (1, 2, 3, etc.) months ago

    >>> n_months_ago_date_string(12, '2016-02-29')
    '2015-02-28'

    >>> n_months_ago_date_string(12, '2015-08-31')
    '2014-08-31'

    >>> n_months_ago_date_string(2, '2015-08-31')
    '2015-06-30'

    >>> n_months_ago_date_string(3, '2014-03-11')
    '2013-12-11'

    >>> n_months_ago_date_string(5, '2014-03-11')
    '2013-10-11'
    """
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return n_months_ago_date(n, today=today).strftime('%Y-%m-%d')

def n_weeks_ago_date(n, today=datetime.date.today()):
    return (today - relativedelta(weeks=n))

def n_weeks_ago_date_string(n, today=datetime.date.today()):
    """Returns a string representing n (1, 2, 3, etc.) weeks ago

    >>> n_weeks_ago_date_string(1, '2016-02-29')
    '2016-02-22'

    >>> n_weeks_ago_date_string(2, '2015-08-31')
    '2015-08-17'

    >>> n_weeks_ago_date_string(3, '2015-08-31')
    '2015-08-10'

    >>> n_weeks_ago_date_string(4, '2014-03-11')
    '2014-02-11'

    >>> n_weeks_ago_date_string(5, '2014-03-11')
    '2014-02-04'
    """
    if isinstance(today, str):
        today = datetime.datetime.strptime(today,"%Y-%m-%d").date()
    return n_weeks_ago_date(n, today=today).strftime('%Y-%m-%d')

def remove_weekends(data):
    """ Return Series/DataFrame with weekend dates removed """
    # http://pandas.pydata.org/pandas-docs/stable/missing_data.html
    return data[data.index.weekday < 5]

def remove_empty_rows(data):
    """ Return Series/DataFrame with only rows containing valid data """
    # http://pandas.pydata.org/pandas-docs/stable/missing_data.html
    return data.dropna(axis=0)

def df_stale_days(dataframe):
    """ Return integer number of calendar days since last observation
        according to dataframe index """
    staleness = datetime.datetime.now() - dataframe.index.to_pydatetime()[-1]
    return staleness.days

def plot_many_against_one(one, many, dataframe, title=None):
    """
    Displays array of plots, each with y-axis dataframe[one] and x-axis dataframe[name]
    where "name" is a value in the list "many."
    
    If there are four or fewer values in "many," it will print four plots in a single row.
    If there are more than 16 values in "many," it will print in rows of four.
    If there are 5-15 values in "many," it will choose a nice number of rows/columns.     
    """

    fig = plt.figure()

    import math

    num_plots = len(many)

    
    if num_plots <= 4:
        per_row = num_plots
    elif math.ceil(math.sqrt(num_plots)) >= 4:
        per_row = 4
    else:
        per_row = math.ceil(math.sqrt(num_plots))
    rows = math.ceil(num_plots / per_row)

    #axes1 = fig.add_axes([0.1/num_plots * 1 + 0.3 * 0, 0.1, 0.9/num_plots, 0.9]) # main axes
    #axes2 = fig.add_axes([0.1/num_plots * 2 + 0.3 * 1, 0.1, 0.9/num_plots, 0.9]) # inset axes
    #axes3 = fig.add_axes([0.1/num_plots * 3 + 0.3 * 2, 0.1, 0.9/num_plots, 0.9]) # inset axes

    x_width = 0.9
    y_height = 1 - 0.1 * rows

    x_padding = 1 - x_width
    plot_width = x_width / per_row
    y_padding = 1 - y_height
    y_row_padding = y_padding / rows
    plot_height = (y_height - y_padding) / rows

    if title: fig.suptitle(title)
    for row in range(1, rows + 1):
        for col in range(1, per_row + 1):
            i = (row - 1) * per_row + col
            x_offset = x_padding/per_row * (col - 1) + plot_width * (col - 1)
            y_offset = y_padding/rows * (rows - row) + plot_height * (rows - row)
            globals()['axes' + str(i)] = fig.add_axes([x_offset, y_offset, plot_width, plot_height])
            globals()['axes' + str(i)].scatter(dataframe[many[i - 1]], dataframe[one], c='g', alpha=0.1)
            globals()['axes' + str(i)].set_xlabel(many[i - 1])
            globals()['axes' + str(i)].set_title(many[i - 1] + ' vs. ' + one)
            if col == 1: globals()['axes' + str(i)].set_ylabel(one)
            if i == num_plots: break

if __name__ == '__main__':
    import doctest
    doctest.testmod()



from setuptools import setup, find_packages

setup(
      name = 'hedgeye_analysis',
      version = '0.1.8',
      description = 'Python analysis tools used by Hedgeye',
      url = 'https://github.com/hedgeyedev/hedgeye-python-analysis/hedgeye_analysis',
      author='James Lavin',
      author_email='jlavin@hedgeye.com',
      license='MIT',
      packages=find_packages(exclude=['test','test.*']),
      install_requires=[
          'IPython',
          'matplotlib',
          'numpy',
          'pandas',
          'seaborn',
      ],
      test_suite='nose.collector',
      classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4'
      ],
      zip_safe=False
      )
